@@ -1,8 +1,8 @@
 "summary.radfit.frame" <-
-function (object, ...) 
+function (object, ...)
 {
     labels <- names(object)
-    for (i in 1:length(labels)) {
+    for (i in seq_along(labels)) {
         cat("\n***", labels[i], "***\n")
         print(object[[i]], ...)
     }
